package de.intevation.flys.artifacts.states.fixation;

import java.text.DateFormat;

import de.intevation.artifactdatabase.state.Facet;

import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.FixationArtifactAccess;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.ReportFacet;

import de.intevation.flys.artifacts.model.fixings.FixCalculation;

import de.intevation.flys.artifacts.model.fixings.FixAvSectorFacet;
import de.intevation.flys.artifacts.model.fixings.FixAnalysisEventsFacet;
import de.intevation.flys.artifacts.model.fixings.FixAnalysisPeriodsFacet;
import de.intevation.flys.artifacts.model.fixings.FixReferenceEventsFacet;
import de.intevation.flys.artifacts.model.fixings.FixOutlierFacet;
import de.intevation.flys.artifacts.model.fixings.FixWQCurveFacet;
import de.intevation.flys.artifacts.model.fixings.FixDerivateFacet;
import de.intevation.flys.artifacts.model.fixings.FixDeviationFacet;
import de.intevation.flys.artifacts.model.fixings.FixLongitudinalAnalysisFacet;
import de.intevation.flys.artifacts.model.fixings.FixLongitudinalDeviationFacet;
import de.intevation.flys.artifacts.model.fixings.FixLongitudinalReferenceFacet;

import de.intevation.flys.artifacts.model.fixings.DateRange;

import de.intevation.flys.artifacts.states.DefaultState;

import de.intevation.flys.artifacts.resources.Resources;

import java.util.List;

import org.apache.log4j.Logger;

/**
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixationCompute
extends      DefaultState
implements   FacetTypes
{

    /** The log used in this class. */
    private static Logger log = Logger.getLogger(FixationCompute.class);

    private static final String I18N_REFERENCEPERIOD = "fix.reference.period.events";

    private static final String I18N_ANALYSISPERIODS = "fix.analysis.periods";

    private static final String I18N_DERIVATIVE = "fix.derivative";

    private static final String I18N_OUTLIER = "fix.outlier";

    private static final String I18N_ANALYSIS = "fix.analysis";

    private static final String I18N_DEVIATION = "fix.deviation";

    private static final String I18N_REFERENCEDEVIATION = "fix.reference.deviation";

    /**
     * The default constructor that initializes an empty State object.
     */
    public FixationCompute() {
    }

    @Override
    public Object computeAdvance(
        FLYSArtifact artifact,
        String       hash,
        CallContext  context,
        List<Facet>  facets,
        Object       old
    ) {
        log.debug("FixationCompute.computeAdvance");

        CalculationResult res;

        FixationArtifactAccess access =
            new FixationArtifactAccess(artifact);

        if (old instanceof CalculationResult) {
            res = (CalculationResult)old;
        }
        else {
            FixCalculation calc =
                new FixCalculation(access);

            res = calc.calculate();
        }

        if (facets == null) {
            return res;
        }

        if (res.getReport().hasProblems()) {
            facets.add(new ReportFacet(ComputeType.ADVANCE, hash, id));
        }
        facets.add(
            new DataFacet(CSV, "CSV data", ComputeType.ADVANCE, hash, id));

        int count = access.getAnalysisPeriods().length;
        for (int i = 0; i < count; i++) {
            DateRange period = access.getAnalysisPeriods()[i];
            DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM);
            String startDate = df.format(period.getFrom());
            String endDate = df.format(period.getTo());

            for (int j = access.getQSectorStart().intValue();
                 j <= access.getQSectorEnd().intValue();
                 j++) {

                String sector = "";
                switch (j) {
                    case 0: sector = "[0 - (MNQ+MQ)/2)"; break;
                    case 1: sector = "[(MNQ+MQ)/2 - (MQ+MHQ)/2)"; break;
                    case 2: sector = "[(MQ+MHQ)/2 - HQ5)"; break;
                    case 3: sector = "[HQ5 - \u221e)"; break;
                }
                String description = sector + ": " +
                                     startDate + " - " +
                                     endDate;

                int sectorNdx = j - access.getQSectorStart().intValue();
                int facetNdx = i << 2;
                facetNdx = facetNdx | j;

                facets.add(
                    new FixAvSectorFacet(facetNdx,
                                         FIX_SECTOR_AVERAGE_DWT + "_" + sectorNdx,
                                         description));
                facets.add(
                    new FixLongitudinalAnalysisFacet(facetNdx,
                                         FIX_SECTOR_AVERAGE_LS + "_" + sectorNdx,
                                         description));
                String dev = "Abweichung: " + description;
                facets.add(
                    new FixLongitudinalAnalysisFacet(facetNdx,
                                         FIX_SECTOR_AVERAGE_LS_DEVIATION + "_" + sectorNdx,
                                         dev));
                facets.add(
                    new FixAvSectorFacet(facetNdx,
                                         FIX_SECTOR_AVERAGE_WQ + "_" + sectorNdx,
                                         description));

            }
            String eventDesc =
                Resources.getMsg(context.getMeta(),
                                 I18N_ANALYSIS,
                                 I18N_ANALYSIS) +
                ": " + startDate + " - " + endDate;
            facets.add(new FixAnalysisEventsFacet(i,
                FIX_ANALYSIS_EVENTS_DWT,
                eventDesc));
            facets.add(new FixLongitudinalAnalysisFacet(i,
                FIX_ANALYSIS_EVENTS_LS,
                eventDesc));
            facets.add(new FixAnalysisEventsFacet(i,
                FIX_ANALYSIS_EVENTS_WQ,
                eventDesc));
        }

        String i18n_ref = Resources.getMsg(context.getMeta(),
                I18N_REFERENCEPERIOD,
                I18N_REFERENCEPERIOD);
        String i18n_dev = Resources.getMsg(context.getMeta(),
                I18N_REFERENCEDEVIATION,
                I18N_REFERENCEDEVIATION);


        facets.add(new FixReferenceEventsFacet(0,
            FIX_REFERENCE_EVENTS_DWT,
            i18n_ref));
        facets.add(new FixLongitudinalReferenceFacet(1,
            FIX_REFERENCE_EVENTS_LS,
            i18n_ref));
        facets.add(new FixReferenceEventsFacet(2,
            FIX_REFERENCE_EVENTS_WQ,
            i18n_ref));

        facets.add(new FixLongitudinalDeviationFacet(0,
            FIX_DEVIATION_LS,
            i18n_dev));

        String i18n_ana = Resources.getMsg(context.getMeta(),
                I18N_ANALYSISPERIODS,
                I18N_ANALYSISPERIODS);
        facets.add(new FixAnalysisPeriodsFacet(0,
            FIX_ANALYSIS_PERIODS_DWT,
            i18n_ana));
        facets.add(new FixAnalysisPeriodsFacet(1,
            FIX_ANALYSIS_PERIODS_LS,
            i18n_ana));
        facets.add(new FixAnalysisPeriodsFacet(2,
            FIX_ANALYSIS_PERIODS_WQ,
            i18n_ana));

        facets.add(new FixWQCurveFacet("W/Q"));
        facets.add(new FixOutlierFacet(FIX_OUTLIER,
            Resources.getMsg(context.getMeta(), I18N_OUTLIER, I18N_OUTLIER)));
        facets.add(new FixDerivateFacet(FIX_DERIVATE,
            Resources.getMsg(context.getMeta(),
                I18N_DERIVATIVE,
                I18N_DERIVATIVE)));

        facets.add(new FixDeviationFacet(FIX_DEVIATION_DWT,
            Resources.getMsg(context.getMeta(),
                I18N_DEVIATION,
                I18N_DEVIATION)));
        return res;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
